#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <sys/time.h>
#include "ch37x_lib.h"

#define MB (1024 * 1024)

#define BULKLEN	64
#define INTLEN	8

#define DEBUG

char choice[16];
char choicenum;

static unsigned char bulk_in_num = 0;
static unsigned char bulk_out_num = 0;
static unsigned char interrupt_in_num = 0;
static unsigned char interrupt_out_num = 0;

unsigned char bulk_w_buf[BULKLEN];
unsigned char bulk_r_buf[8*BULKLEN];
unsigned char int_w_buf[BULKLEN];
unsigned char int_r_buf[8*BULKLEN];

void Show_DevMsg(void)
{
	char *version = malloc(sizeof(char) * 30);
	unsigned int dev_id;
	unsigned int ep_num;
	
	printf("\n");
	printf("Information of %s lists below:\n", (char *)CH37XGetDeviceName(0));
	printf("**************************************\n");
	CH37XGetDriverVersion(0, version);
	CH37XGetDeviceID(0, &dev_id);
	CH37XGetDeviceMsg(0, &ep_num);

	bulk_in_num = (char)(ep_num>>24);
	bulk_out_num = (char)(ep_num>>16);
	interrupt_in_num = (char)(ep_num>>8);
	interrupt_out_num = (char)ep_num;

	printf("Driver version: %s\n", version);
	printf("Vendor ID: 0x%4x, Product ID: 0x%4x\n", (unsigned short)dev_id, (unsigned short)(dev_id>>16));
	printf("bulk_in_num: %d\n", bulk_in_num);
	printf("bulk_out_num: %d\n", bulk_out_num);
	printf("interrupt_in_num: %d\n", interrupt_in_num);
	printf("interrupt_out_num: %d\n", interrupt_out_num);
}

static int Bulk_Up(int epnum)
{
	bool ret = false;
	int len = sizeof(bulk_r_buf);
	unsigned long bytes_read = BULKLEN;
	
	ret = CH37XReadData(0, epnum, bulk_r_buf, &bytes_read);
	if (ret == false) {
		printf("CH37XReadData error!");
		return -1;
	}
	
#ifdef DEBUG
    printf("Buffer read: %ld\n", bytes_read);
	for (int i = 0; i < bytes_read; i++)
	   printf("0x%.2x ", bulk_r_buf[i]);
    printf("\n");
#endif

	return bytes_read;
}

static void Speed_Test_Read(int num, int MB_Num)
{
	unsigned long bytes_read = 0;
	struct timeval t1, t2;
	int delta_sec = 0;
	int delta_usec = 0;
	float speed_read = 0.0;

recycle:
	bytes_read = 0;
	gettimeofday(&t1, NULL);
	while(bytes_read <= MB_Num * MB) 
		bytes_read += Bulk_Up(num);
	gettimeofday(&t2, NULL);
	
	delta_sec = t2.tv_sec - t1.tv_sec; 
	delta_usec = t2.tv_usec - t1.tv_usec;

	if(delta_usec < 0) {
		delta_sec --;
		delta_usec = 1000000 + delta_usec;
	}
	speed_read = (float)(MB_Num) / (float)(delta_sec + (float)delta_usec / 1000000); 

	printf("%d MB takes: %2d sec %2d us, up speed: %4.2f MB/s\n", MB_Num, delta_sec, delta_usec, speed_read);

	goto recycle;
}

static int Bulk_Down(int epnum)
{
	bool ret = false;
	int len = sizeof(bulk_w_buf);
	unsigned long bytes_write = BULKLEN;

	for (int i = 0; i < BULKLEN; i++)
		bulk_w_buf[i] = i;
	
	ret = CH37XWriteData(0, epnum, bulk_w_buf, &bytes_write);
	if (ret == false) {
		printf("CH37XWriteData error!\n");
	}
	printf("wrote %ld bytes already.", bytes_write);
	
	return bytes_write;
}

static void Speed_Test_Write(int num, int MB_Num)
{
	unsigned long bytes_write = 0;
	struct timeval t1, t2;
	int delta_sec = 0;
	int delta_usec = 0;
	float speed_write = 0.0;

recycle:
	bytes_write = 0;
	gettimeofday(&t1, NULL);
	while(bytes_write <= MB_Num * MB) 
		bytes_write += Bulk_Down(num);
	gettimeofday(&t2, NULL);
	
	delta_sec = t2.tv_sec - t1.tv_sec; 
	delta_usec = t2.tv_usec - t1.tv_usec;

	if(delta_usec < 0) {
		delta_sec --;
		delta_usec = 1000000 + delta_usec;
	}
	speed_write = (float)(MB_Num) / (float)(delta_sec + (float)delta_usec / 1000000); 

	printf("%d MB takes: %2d sec %2d us, down speed: %4.2f MB/s\n", MB_Num, delta_sec, delta_usec, speed_write);

	goto recycle;
}


static int Int_Up(int num)
{
	printf("Not implemented yet.\n");
	return 0;
}

static int Int_Down(int num)
{
	printf("Not implemented yet.\n");
	return 0;
}

void Show_Choice(void)
{
	int i = 0;
	printf("************************\n");
	printf("0-->Exit\n");
	for( i = 0; i < choicenum; i++ ) {
		switch(choice[i]) {
		case 1: 
			printf("1-->BULK0_UP\n");
			break;
		case 2:
			printf("2-->BULK0_DOWN\n");
			break;
		case 3:
			printf("3-->INT0_UP\n");
			break;
		case 4:
			printf("4-->INT0_DOWN\n");
			break;
		case 5:
			printf("5-->BULK1_UP\n");
			break;
		case 6:
			printf("6-->BULK1_DOWN\n");
			break;
		case 7:
			printf("7-->INT1_UP\n");
			break;
		case 8:
			printf("8-->INT1_DOWN\n");
			break;
		default:
			break;
		}
	}
}

void Show_Menu(void)
{
	char i = 0;
	choicenum = 0;
	
	memset(choice, 0, sizeof(choice));	
	if(bulk_in_num >= 1) {
		choice[i++] = 1; 
		if(bulk_in_num >= 2) {
			choice[i++] = 5;
		}
	}
	if(bulk_out_num >= 1) {
		choice[i++] = 2; 
		if(bulk_out_num >= 2) {
			choice[i++] = 6;
		}
	}
	if(interrupt_in_num >= 1) {
		choice[i++] = 3; 
		if(interrupt_in_num >= 2) {
			choice[i++] = 7;
		}
	}
	if(interrupt_out_num >= 1) {
		choice[i++] = 4; 
		if(interrupt_out_num >= 2) {
			choice[i++] = 8;
		}
	}
	choicenum = i;
	Show_Choice();
}

static char test_num(const char *buf, char len, int num)
{
	char i = 0;
	
	for( i = 0; i < len; i++ ) {
		if(num == *(buf + i))
			return 1;
	}
	return 0;
}

void menu(void)
{
	int num = 0;
	int i = 0;
	
	Show_Menu();
	while(1) {
		printf("************************\n");
		printf("Please input number for test:\n");
		scanf("%d", &num);
		if(!test_num(choice, choicenum, num) && (num != 0)) {
			printf("Bad choice, try again!\n");
			Show_Choice();
			continue;
		}
		switch (num) {
		case 0:
			printf("Test over!\n");
			return;
		case 1:
		case 5:
			Bulk_Up(num / 4);
			break;
		case 2:
		case 6:
			Bulk_Down(num / 4);
			break;
		case 3:
		case 7:
			Int_Up(num / 4);
			break;
		case 4:
		case 8:
			Int_Down(num / 4 - 1);
			break;
		default:
			printf("Bad choice, try again!\n");
			break;
		}
	}
}

int main(int argc, char *argv[])
{
	int devnum = 0;
	bool ret;
	
	devnum = CH37XScanDevices();
	printf("Scan %d devices.\n", devnum);
	
	if (devnum == 0)
		return 0;
	
	ret = CH37XOpenDevice(0);
	if (ret == false)
		return -1;
		
	ret = CH37XSetTimeout(0, 3000, 3000);
	if (ret == false)
		return -1;

	ret = CH37XSetBufUpload(0, 0, true);
	
	Show_DevMsg();
	
	sleep(3);
//	Speed_Test_Read(1, 100);
//	Speed_Test_Write(2, 100);
	menu();
	
	ret = CH37XCloseDevice(0);
	
	return 0;
}


